unit DBUpdate;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Db, DBTables;

type
  EDBUpdateErr = class(Exception);


TDBUpdate = class(TComponent)
     protected
       SQLSentence : string;       // Compone la sentencia SQL
     public
       madequery : TQuery;
       FTableName   : string;
       constructor Create(AOwner: TComponent); override;
       destructor  Destroy;override;
       procedure Clear;
       procedure getfromTable(Table : TTable);
       procedure AddField( const Name: string; DataType: TFieldType; Size: Word);
       procedure KillField( const Name: string );
       function  getSQLSentence : string;
       procedure Exec;      // Ejecuta la sentencia
  published
       Property TableName : string read FTableName write FTableName ;
end;

procedure Register;

implementation

{
   Sample test to see how you insert a Field and then you kill them
  
procedure TForm1.Button1Click(Sender: TObject);
var
  DBTest : TDBUpdate;
begin
  DBTest := TDBUpdate.Create(Self);
  DBTest.Clear;
  DBTest.TableName := 'c:\temp\quot-lin.db';
  DBTest.AddField('OT Time', ftInteger, 0);
  DBTest.KillField('Text');
  DBTest.Exec;	// Process it
  DBTest.Free;
end;
}

{ Create the query memory }
constructor TDBUpdate.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  madequery := TQuery.Create(Self);
end;

{ frees the memory }
destructor TDBUpdate.Destroy;
begin
//  madequery.Destroy;
  inherited Destroy;
end;

{   Reiniciar las variables
}
procedure TDBUpdate.Clear;
begin
  FTableName  := '';
  SQLSentence := '';       // Compone la sentencia SQL
end;

{ Add a filter }
procedure TDBUpdate.AddField( const Name: string; DataType: TFieldType; Size: Word);
var
 sentence : string;
begin
 sentence := '';
 // Check some raisings....
 if Name = '' then
     raise EDBUpdateErr.Create('Blank field name inserted');
 if Pos(' ', Name) <> 0 then
     raise EDBUpdateErr.Create('The field name cannot have spaces');
 { Depende del tipo, aadir la instruccin }
 if DataType = ftString then
      sentence := 'ADD '+Name+' Char('+ IntToStr( Size ) + ')';
 if DataType = ftInteger then
      sentence := 'ADD '+Name+' Integer';
 if DataType = ftSmallInt then
      sentence := 'ADD '+Name+' SmallInt';
 if DataType = ftFloat then
      sentence := 'ADD '+Name+' Float('+ IntToStr( Size ) +',0)';
 if DataType = ftDate then
      sentence := 'ADD '+Name+' Date';
 if DataType = ftTime then
      sentence := 'ADD '+Name+' Time';
 if DataType = ftDateTime then
      sentence := 'ADD '+Name+' TimeStamp';
 if sentence = '' then
     raise EDBUpdateErr.Create('DataType not recognised');
 // Ir aadiendo la instruccin
 if SQLSentence = '' then
      SQLSentence := sentence
    else
      SQLSentence := SQLSentence + ', ' + sentence;
end;

{ Kill a field from a table }
procedure TDBUpdate.KillField( const Name: string );
begin
 if Pos(' ', Name) <> 0 then
     raise EDBUpdateErr.Create('The field name cannot have spaces');
 if SQLSentence = '' then
      SQLSentence := 'DROP ' + Name
    else
      SQLSentence := SQLSentence + ', DROP ' + Name;
end;

{ Return the SQL Sentence }
function TDBUpdate.getSQLSentence : string;
begin
  Result := 'ALTER TABLE "'+ FTableName +'" ' + SQLSentence + ';';
end;

{   From a structured tables, get the name
}
procedure TDBUpdate.getfromTable(Table : TTable);
begin
  FTableName := Table.TableName;
  if Table.Active then Table.Close;
end;

{   Caa al mono
    try to execute the query
}
procedure TDBUpdate.Exec;      // Ejecuta la sentencia
begin
  try
    // Unidirectional an application requires less memory and performance is improved.
    madequery.UniDirectional := true;
    madequery.SQL.Clear;
    madequery.SQL.Add( getSQLSentence );
    madequery.ExecSQL;
  except
    raise;
  end;
end;

{-register components}
procedure Register;
begin
  RegisterComponents('Data Access', [TDBUpdate]);
end;

end.
